/*
 * Copyright notice from original mutt:
 * Copyright (C) 2000-5 Brendan Cully <brendan@kublai.com>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

/* remote host account manipulation (POP/IMAP) */

#include <lib-lib/lib-lib.h>
#include <lib-ui/lib-ui.h>

#include "mutt.h"
#include "account.h"

/* mutt_account_match: compare account info (host/port/user/login) */
int mutt_account_match (const ACCOUNT * a1, const ACCOUNT * a2)
{
    const char* user  = NONULL(mod_core.username);

    if (a1->type != a2->type)
        return 0;
    if (ascii_strcasecmp (a1->host, a2->host))
        return 0;
    if (a1->port != a2->port)
        return 0;

    if (a1->type == M_ACCT_TYPE_IMAP && !m_strisempty(ImapUser))
        user = ImapUser;

    if (a1->type == M_ACCT_TYPE_POP && !m_strisempty(PopUser))
        user = PopUser;

    if (a1->has_user && a2->has_user)
        return !m_strcmp(a1->user, a2->user);
    if (a1->has_user)
        return !m_strcmp(a1->user, user);
    if (a2->has_user)
        return !m_strcmp(a2->user, user);

    return 1;
}

/* mutt_account_fromurl: fill account with information from url. */
int mutt_account_fromurl(ACCOUNT *account, ciss_url_t *url)
{
    /* must be present */
    if (!url->host)
        return -1;
    m_strcpy(account->host, sizeof(account->host), url->host);

    if (url->user) {
        m_strcpy(account->user, sizeof(account->user), url->user);
        account->has_user = 1;
    }
    if (url->pass) {
        m_strcpy(account->pass, sizeof(account->pass), url->pass);
        account->has_pass = 1;
    }
    if (url->port) {
        account->port = url->port;
        account->has_port = 1;
    }

    return 0;
}

/* mutt_account_tourl: fill URL with info from account. The URL information
 *   is a set of pointers into account - don't free or edit account until
 *   you've finished with url (make a copy of account if you need it for
 *   a while). */
void mutt_account_tourl(ACCOUNT *account, ciss_url_t *url)
{
    url->scheme = U_UNKNOWN;
    url->user = NULL;
    url->pass = NULL;
    url->port = 0;

    if (account->type == M_ACCT_TYPE_IMAP) {
        url->scheme = account->has_ssl ? U_IMAPS : U_IMAP;
    }

    if (account->type == M_ACCT_TYPE_POP) {
        url->scheme = account->has_ssl ? U_POPS : U_POP;
    }

    url->host = account->host;
    if (account->has_port)
        url->port = account->port;
    if (account->has_user)
        url->user = account->user;
    if (account->has_pass)
        url->pass = account->pass;
}

/* mutt_account_getuser: retrieve username into ACCOUNT, if necessary */
int mutt_account_getuser (ACCOUNT * account)
{
    char prompt[STRING];

    /* already set */
    if (account->has_user)
        return 0;
    else if ((account->type == M_ACCT_TYPE_IMAP) && !m_strisempty(ImapUser))
        m_strcpy(account->user, sizeof(account->user), ImapUser);
    else if ((account->type == M_ACCT_TYPE_POP) && !m_strisempty(PopUser))
        m_strcpy(account->user, sizeof(account->user), PopUser);
    /* prompt (defaults to unix username), copy into account->user */
    else {
        snprintf(prompt, sizeof(prompt), _("Username at %s: "), account->host);
        m_strcpy(account->user, sizeof(account->user), NONULL(mod_core.username));
        if (mutt_get_field_unbuffered(prompt, account->user,
                                      sizeof(account->user), 0))
            return -1;
    }

    account->has_user = 1;

    return 0;
}

int mutt_account_getlogin (ACCOUNT* account)
{
    /* already set */
    if (account->has_login)
        return 0;

    if (account->type == M_ACCT_TYPE_IMAP && !m_strisempty(ImapLogin)) {
        m_strcpy(account->login, sizeof(account->login), ImapLogin);
        account->has_login = 1;
    } else {
        mutt_account_getuser (account);
        m_strcpy(account->login, sizeof(account->login), account->user);
    }

    account->has_login = 1;

    return 0;
}

/* mutt_account_getpass: fetch password into ACCOUNT, if neccessary */
int mutt_account_getpass (ACCOUNT * account)
{
    char prompt[STRING];

    if (account->has_pass)
        return 0;
    else if ((account->type == M_ACCT_TYPE_IMAP) && !m_strisempty(ImapPass))
        m_strcpy(account->pass, sizeof(account->pass), ImapPass);
    else if ((account->type == M_ACCT_TYPE_POP) && !m_strisempty(PopPass))
        m_strcpy(account->pass, sizeof(account->pass), PopPass);
    else {
        snprintf(prompt, sizeof(prompt), _("Password for %s@%s: "),
                 account->has_login ? account->login : account->user,
                 account->host);
        account->pass[0] = '\0';
        if (mutt_get_field_unbuffered(prompt, account->pass,
                                      sizeof(account->pass), M_PASS))
            return -1;
    }

    account->has_pass = 1;

    return 0;
}
